#!/usr/bin/node

import * as tf from '@tensorflow/tfjs-node'
import * as fs from 'fs'
import * as jpeg from 'jpeg-js'

const mysteries = [
	'/var/lib/pv/mystery1.jpg',
	'/var/lib/pv/mystery2.jpg',
];

import { INCEPTION_CLASSES } from './labels.mjs'

tf.ready().then(() => {
	const modelPath =
		'https://tfhub.dev/google/tfjs-model/imagenet/inception_v3/classification/3/default/1'
	tf.tidy(() => {
		tf.loadGraphModel(modelPath, { fromTFHub: true }).then((model) => {
			for (let n = 0; n < mysteries.length; n++) {
				const mysteryImage = jpeg.decode(fs.readFileSync(mysteries[n]))
				const myTensor = tf.browser.fromPixels(mysteryImage)
				// Inception v3 expects an image resized to 299x299
				const readyfied = tf.image
					.resizeBilinear(myTensor, [299, 299], true)
					.div(255)
					.reshape([1, 299, 299, 3])

				const result = model.predict(readyfied)
				result.print() // useless

				const { values, indices } = tf.topk(result, 3)
				indices.print()

				// Let's hear those winners
				const winners = indices.dataSync()
				console.log(`
					🥇 First place ${INCEPTION_CLASSES[winners[0]]},
					🥈 Second place ${INCEPTION_CLASSES[winners[1]]},
					🥉 Third place ${INCEPTION_CLASSES[winners[2]]}
				`)
			}
		})
	})
})

